@@ -122,7 +122,13 @@ transfer.  For example:
 
 
 """
-from botocore.exceptions import ClientError
+
+import logging
+import threading
+from os import PathLike, fspath, getpid
+
+from botocore.compat import HAS_CRT
+from botocore.exceptions import ClientError, MissingDependencyException
 from s3transfer.exceptions import (
     RetriesExceededError as S3TransferRetriesExceededError,
 )
@@ -132,11 +138,23 @@ from s3transfer.manager import TransferManager
 from s3transfer.subscribers import BaseSubscriber
 from s3transfer.utils import OSUtils
 
-from boto3.exceptions import RetriesExceededError, S3UploadFailedError
+import boto3.s3.constants as constants
+from boto3.compat import TRANSFER_CONFIG_SUPPORTS_CRT
+from boto3.exceptions import (
+    RetriesExceededError,
+    S3UploadFailedError,
+)
+
+if HAS_CRT:
+    import awscrt.s3
+
+    from boto3.crt import create_crt_transfer_manager
 
 KB = 1024
 MB = KB * KB
 
+logger = logging.getLogger(__name__)
+
 
 def create_transfer_manager(client, config, osutil=None):
     """Creates a transfer manager based on configuration
@@ -153,6 +171,73 @@ def create_transfer_manager(client, config, osutil=None):
     :rtype: s3transfer.manager.TransferManager
     :returns: A transfer manager based on parameters provided
     """
+    if _should_use_crt(config):
+        crt_transfer_manager = create_crt_transfer_manager(client, config)
+        if crt_transfer_manager is not None:
+            logger.debug(
+                f"Using CRT client. pid: {getpid()}, thread: {threading.get_ident()}"
+            )
+            return crt_transfer_manager
+
+    # If we don't resolve something above, fallback to the default.
+    logger.debug(
+        f"Using default client. pid: {getpid()}, thread: {threading.get_ident()}"
+    )
+    return _create_default_transfer_manager(client, config, osutil)
+
+
+def _should_use_crt(config):
+    # This feature requires awscrt>=0.19.18
+    has_min_crt = HAS_CRT and has_minimum_crt_version((0, 19, 18))
+    is_optimized_instance = has_min_crt and awscrt.s3.is_optimized_for_system()
+    pref_transfer_client = config.preferred_transfer_client.lower()
+
+    if (
+        pref_transfer_client == constants.CRT_TRANSFER_CLIENT
+        and not has_min_crt
+    ):
+        msg = (
+            "CRT transfer client is configured but is missing minimum CRT "
+            f"version. CRT installed: {HAS_CRT}"
+        )
+        if HAS_CRT:
+            msg += f", with version: {awscrt.__version__}"
+        raise MissingDependencyException(msg=msg)
+
+    if (
+        is_optimized_instance
+        and pref_transfer_client == constants.AUTO_RESOLVE_TRANSFER_CLIENT
+    ) or pref_transfer_client == constants.CRT_TRANSFER_CLIENT:
+        logger.debug(
+            "Attempting to use CRTTransferManager. Config settings may be ignored."
+        )
+        return True
+
+    logger.debug(
+        "Opting out of CRT Transfer Manager. Preferred client: "
+        f"{pref_transfer_client}, CRT available: {HAS_CRT}, "
+        f"Instance Optimized: {is_optimized_instance}."
+    )
+    return False
+
+
+def has_minimum_crt_version(minimum_version):
+    """Not intended for use outside boto3."""
+    if not HAS_CRT:
+        return False
+
+    crt_version_str = awscrt.__version__
+    try:
+        crt_version_ints = map(int, crt_version_str.split("."))
+        crt_version_tuple = tuple(crt_version_ints)
+    except (TypeError, ValueError):
+        return False
+
+    return crt_version_tuple >= minimum_version
+
+
+def _create_default_transfer_manager(client, config, osutil):
+    """Create the default TransferManager implementation for s3transfer."""
     executor_cls = None
     if not config.use_threads:
         executor_cls = NonThreadedExecutor
@@ -164,17 +249,31 @@ class TransferConfig(S3TransferConfig):
         'max_concurrency': 'max_request_concurrency',
         'max_io_queue': 'max_io_queue_size',
     }
+    DEFAULTS = {
+        'multipart_threshold': 8 * MB,
+        'max_concurrency': 10,
+        'max_request_concurrency': 10,
+        'multipart_chunksize': 8 * MB,
+        'num_download_attempts': 5,
+        'max_io_queue': 100,
+        'max_io_queue_size': 100,
+        'io_chunksize': 256 * KB,
+        'use_threads': True,
+        'max_bandwidth': None,
+        'preferred_transfer_client': constants.AUTO_RESOLVE_TRANSFER_CLIENT,
+    }
 
     def __init__(
         self,
-        multipart_threshold=8 * MB,
-        max_concurrency=10,
-        multipart_chunksize=8 * MB,
-        num_download_attempts=5,
-        max_io_queue=100,
-        io_chunksize=256 * KB,
-        use_threads=True,
+        multipart_threshold=None,
+        max_concurrency=None,
+        multipart_chunksize=None,
+        num_download_attempts=None,
+        max_io_queue=None,
+        io_chunksize=None,
+        use_threads=None,
         max_bandwidth=None,
+        preferred_transfer_client=None,
     ):
         """Configuration object for managed S3 transfers
 
@@ -185,7 +284,7 @@ class TransferConfig(S3TransferConfig):
         :param max_concurrency: The maximum number of threads that will be
             making requests to perform a transfer. If ``use_threads`` is
             set to ``False``, the value provided is ignored as the transfer
-            will only ever use the main thread.
+            will only ever use the current thread.
 
         :param multipart_chunksize: The partition size of each part for a
             multipart transfer.
@@ -198,39 +297,73 @@ class TransferConfig(S3TransferConfig):
             Other retryable exceptions such as throttling errors and 5xx
             errors are already retried by botocore (this default is 5). This
             does not take into account the number of exceptions retried by
-            botocore.
+            botocore. Note: This value is ignored when resolved transfer
+            manager type is CRTTransferManager.
 
         :param max_io_queue: The maximum amount of read parts that can be
             queued in memory to be written for a download. The size of each
             of these read parts is at most the size of ``io_chunksize``.
+            Note: This value is ignored when resolved transfer manager type
+            is CRTTransferManager.
 
         :param io_chunksize: The max size of each chunk in the io queue.
             Currently, this is size used when ``read`` is called on the
-            downloaded stream as well.
+            downloaded stream as well. Note: This value is ignored when
+            resolved transfer manager type is CRTTransferManager.
 
         :param use_threads: If True, threads will be used when performing
             S3 transfers. If False, no threads will be used in
-            performing transfers: all logic will be ran in the main thread.
+            performing transfers; all logic will be run in the current thread.
+            Note: This value is ignored when resolved transfer manager type is
+            CRTTransferManager.
 
         :param max_bandwidth: The maximum bandwidth that will be consumed
             in uploading and downloading file content. The value is an integer
-            in terms of bytes per second.
+            in terms of bytes per second. Note: This value is ignored when
+            resolved transfer manager type is CRTTransferManager.
+
+        :param preferred_transfer_client: String specifying preferred transfer
+            client for transfer operations.
+
+            Current supported settings are:
+              * auto (default) - Use the CRTTransferManager when calls
+                  are made with supported environment and settings.
+              * classic - Only use the origin S3TransferManager with
+                  requests. Disables possible CRT upgrade on requests.
+              * crt - Only use the CRTTransferManager with requests.
         """
+        init_args = {
+            'multipart_threshold': multipart_threshold,
+            'max_concurrency': max_concurrency,
+            'multipart_chunksize': multipart_chunksize,
+            'num_download_attempts': num_download_attempts,
+            'max_io_queue': max_io_queue,
+            'io_chunksize': io_chunksize,
+            'use_threads': use_threads,
+            'max_bandwidth': max_bandwidth,
+            'preferred_transfer_client': preferred_transfer_client,
+        }
+        resolved = self._resolve_init_args(init_args)
         super().__init__(
-            multipart_threshold=multipart_threshold,
-            max_request_concurrency=max_concurrency,
-            multipart_chunksize=multipart_chunksize,
-            num_download_attempts=num_download_attempts,
-            max_io_queue_size=max_io_queue,
-            io_chunksize=io_chunksize,
-            max_bandwidth=max_bandwidth,
+            multipart_threshold=resolved['multipart_threshold'],
+            max_request_concurrency=resolved['max_concurrency'],
+            multipart_chunksize=resolved['multipart_chunksize'],
+            num_download_attempts=resolved['num_download_attempts'],
+            max_io_queue_size=resolved['max_io_queue'],
+            io_chunksize=resolved['io_chunksize'],
+            max_bandwidth=resolved['max_bandwidth'],
         )
         # Some of the argument names are not the same as the inherited
         # S3TransferConfig so we add aliases so you can still access the
         # old version of the names.
         for alias in self.ALIAS:
-            setattr(self, alias, getattr(self, self.ALIAS[alias]))
-        self.use_threads = use_threads
+            setattr(
+                self,
+                alias,
+                object.__getattribute__(self, self.ALIAS[alias]),
+            )
+        self.use_threads = resolved['use_threads']
+        self.preferred_transfer_client = resolved['preferred_transfer_client']
 
     def __setattr__(self, name, value):
         # If the alias name is used, make sure we set the name that it points
@@ -240,10 +373,33 @@ class TransferConfig(S3TransferConfig):
         # Always set the value of the actual name provided.
         super().__setattr__(name, value)
 
+    def __getattribute__(self, item):
+        value = object.__getattribute__(self, item)
+        if not TRANSFER_CONFIG_SUPPORTS_CRT:
+            return value
+        defaults = object.__getattribute__(self, 'DEFAULTS')
+        if item not in defaults:
+            return value
+        if value is self.UNSET_DEFAULT:
+            return defaults[item]
+        return value
+
+    def _resolve_init_args(self, init_args):
+        resolved = {}
+        for init_arg, val in init_args.items():
+            if val is not None:
+                resolved[init_arg] = val
+            elif TRANSFER_CONFIG_SUPPORTS_CRT:
+                resolved[init_arg] = self.UNSET_DEFAULT
+            else:
+                resolved[init_arg] = self.DEFAULTS[init_arg]
+        return resolved
+
 
 class S3Transfer:
     ALLOWED_DOWNLOAD_ARGS = TransferManager.ALLOWED_DOWNLOAD_ARGS
     ALLOWED_UPLOAD_ARGS = TransferManager.ALLOWED_UPLOAD_ARGS
+    ALLOWED_COPY_ARGS = TransferManager.ALLOWED_COPY_ARGS
 
     def __init__(self, client=None, config=None, osutil=None, manager=None):
         if not client and not manager:
@@ -277,8 +433,10 @@ class S3Transfer:
             :py:meth:`S3.Client.upload_file`
             :py:meth:`S3.Client.upload_fileobj`
         """
+        if isinstance(filename, PathLike):
+            filename = fspath(filename)
         if not isinstance(filename, str):
-            raise ValueError('Filename must be a string')
+            raise ValueError('Filename must be a string or a path-like object')
 
         subscribers = self._get_subscribers(callback)
         future = self._manager.upload(
@@ -309,8 +467,10 @@ class S3Transfer:
             :py:meth:`S3.Client.download_file`
             :py:meth:`S3.Client.download_fileobj`
         """
+        if isinstance(filename, PathLike):
+            filename = fspath(filename)
         if not isinstance(filename, str):
-            raise ValueError('Filename must be a string')
+            raise ValueError('Filename must be a string or a path-like object')
 
         subscribers = self._get_subscribers(callback)
         future = self._manager.download(
